/*#======================================================================
# This source code is public domain, and may be freely distributed,
# modified, and used for any purpose.  IT COMES WITH ABSOLUTELY NO
# WARRANTY OF ANY KIND.
#======================================================================
# _DY_CONTRIB_PGM_:pipetalker.c:Talk to programs via a pipe
#======================================================================
# LICENSE MODULES: toolkits:
#                  programs:
#                  servers:
# AUTHORS: 
# DESCRIPTION:Talk to programs via a pipe. 
#======================================================================*/

#include <stdio.h>
#include <string.h>
#include "dt_smiles.h"
#include "dt_progob.h"
#include "du_pipeio.h"

/*============================================================================
 *  zapseq() - deallocate a sequence of objects
 */
 
static void zapseq(dt_Handle seq)
{
   dt_Handle  ob;

   dt_reset(seq);
   while (NULL_OB != (ob = dt_next(seq)))
      dt_dealloc(ob);
   dt_dealloc(seq);
}

/*============================================================================
 *  printsos() - print sequence of strings to stdout and deallocate them
 */
 
static void printsos(dt_Handle sos)
{
   dt_Handle  sob;
   char      *str;
   int        lens;
 
   dt_reset(sos);
   while (NULL_OB != (sob = dt_next(sos))) {
      str = dt_stringvalue(&lens, sob);
      printf("%.*s\n", lens, str);
      dt_dealloc(sob);
   }
   dt_dealloc(sos);
}

/*============================================================================
 *  main()
 */

main(int argc, char **argv)
{
   dt_Handle prog, msgin, msgout, sob, sos, seq;
   char      buf[MX_PIPEBUF];
   int       i;

   /*** Require some arguments. ***/

   if (1 == argc) {
      fprintf(stderr, "usage: $ %s program [ arguments ]\n", *argv);
      exit(1);
   }

   /*** Move arguments to sos ***/

   sos = dt_alloc_seq();
   for (i = 1; i < argc; i++)
      dt_append(sos, dt_alloc_string(strlen(argv[i]), argv[i])),

   /*** Make program object, exit if unable. ***/

   prog = dt_alloc_program(sos);
   zapseq(sos);

   if (NULL_OB == prog) {
      if (0 != dt_errorworst()) printsos(dt_errors(DX_ERR_NOTE));
      fprintf(stderr, "%s: can't create %s program object\n", *argv, argv[1]);
      exit(1);
   }

   /*** Print welcome message. ***/

   printsos(dt_welcome(prog));

   /*** Repeatedly convert input to output message via dt_converse(). ***/

   while (fgets(buf,MX_PIPEBUF,stdin)) {
      buf[strlen(buf)-1]='\0';
      seq = dt_alloc_seq();
      msgout = dt_alloc_string(strlen(buf), buf);
      dt_append(seq, msgout);
      msgin  = dt_converse(prog, seq);
      if (NULL_OB == msgin) break;
      printsos(msgin);
      zapseq(seq);
   }

   printsos(dt_converse(prog, NULL_OB));
   dt_dealloc(prog);
}
